//   $id: $
//    ___
//   |  _\ A free Lemmings clone
//   |   /_  _ _  ___  _   _  ___ 
//   |  || || \ || _ || |_| ||_ -'
//   |__||_||_\_||_  ||_____||___|
//                _| |
//               |___|
//
//   Copyright (C) 1998 Ingo Ruhnke <grumbel@gmx.de>
//     
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <iostream>
#include <string>
#ifndef WIN32
#include <config.h>
#include <getopt.h>
#else /* !WIN32 */
#include "win32/config.h"
#include "win32/getopt.h"
#endif /* !WIN32 */

#include <ClanLib/clanlib.h>
#include <ClanLib/clanlayer2.h>

#include "Client.hh"
#include "Server.hh"
#include "TrueServer.hh"
#include "Pingus.hh"
#include "algo.hh"
#include "globals.hh"
#include "actions/pingu_actions.hh"
#include "LevelInterrupt.hh"
#include "PingusSound.hh"
#include "GameObj.hh"
#include "Playfield.hh"
#include "PingusError.hh"
#include "FileSelector.hh"
#include "PingusMenu.hh"
#include "PingusMessageBox.hh"
#include "PingusLevelResult.hh"
#include "particles/GroundParticle.hh"

#ifdef WIN32
using namespace std;
#endif /* WIN32 */

PingusMain::PingusMain()
{
  string temp;
  resolution = "640x480";

  temp = string(PACKAGE) + " " + VERSION + " - A free Lemmings clone";
  temp += " - http://pingus.home.pages.de";

  title = strdup(temp.c_str());

  pingus_datadir_set = false;
}

char* 
PingusMain::get_title()
{
  return title;
}

void
PingusMain::read_rc_file(void)
{
  char*    homedir = getenv("HOME");
  string   rcfile;
  char     line[1024];
  char     option[1024];
  char     argument[1024];
  int      lineno = 0;
  char*    pargv[100];
  int      pargc = 1;
  ifstream in;

  pargv[0] = strdup(PACKAGE);

  if (!homedir) {
    rcfile = ".pingus/options";
  } else {
    rcfile = string(homedir) + "/.pingus/options";
  }

  in.open(rcfile.c_str());

  if (!in) {
    cout << "No config file found at: " << rcfile << endl;
  } else {
    cout << "Parsing config file" << endl;
    while(true) {
      in.getline(line, 1023);
      ++lineno;
      if(in.eof()) {
	break;
      }

      remove_comments(line);
      if (!line_empty(line)) {
	if (sscanf(line, "%s %s", option, argument) == 2) {
	  cout << "Option: " << option << " Argument: " << argument << endl;
	  pargv[pargc++] = strdup((string("--") + option).c_str());
	  pargv[pargc++] = strdup(argument);
	} else {
	  cout << "Option: " << option << endl;
	  pargv[pargc++] = strdup((string("--") + option).c_str());
	}
      }
    }

    cout << "Argument Start" << endl;
    for(int i=0; i < pargc; ++i) {
      cout << pargv[i] << endl;
    }
    cout << "Argument End" << endl;

    if (pargc == 1) {
      check_args(pargc, pargv);
    } else { 
      check_args(pargc, pargv);
    }
  }
  // FIXME: Waring memory leak!
}

// check_ars() checks the command line for options and set the
// corresponding global variables to the set values.
void 
PingusMain::check_args(int argc, char* argv[])
{
#ifdef WIN32
  cursor_enabled = true;
#endif
  int c = 0; 
  int option_index = 0;
  optind = 0;

  struct option long_options[] =
  {
    {"enable-music",    no_argument,       0, 'm'},
    {"enable-sound",    no_argument,       0, 's'},
    {"enable-gimmicks", no_argument,       0, 'i'},
    {"enable-cursor",   no_argument,       0, 'c'},
    {"disable-intro",   no_argument,       0, 'n'},
    {"play-demo",       required_argument, 0, 'd'},
    {"record-demo",     required_argument, 0, 'r'},
    {"set-speed",       required_argument, 0, 't'},
    {"datadir",         required_argument, 0, 'd'},
    {"level",           required_argument, 0, 'l'},
    {"help",            no_argument,       0, 'h'},
    {"version",         no_argument,       0, 'V'},
    {"verbose",         required_argument, 0, 'v'},
    {"print-fps",       no_argument,       0, 'b'},
    {"sound-specs",     required_argument, 0, 'S'},
    {"geometry",        required_argument, 0, 'g'},
    {"debug-actions",   no_argument,       0, 129},
    {"fs-preload",      no_argument,       0, 130},
    {"fast",            no_argument,       0, 132},
    {"disable-previews",no_argument,       0, 133}, // FIXME: is the number stuff correct?
    {0, 0, 0, 0}
  };

  while(true) {
    c = getopt_long(argc, argv, "r:p:smv:d:l:hVp:bxS:g:it:c", long_options, &option_index);
    
    if (c == -1 || c == 1)
      break;

    switch(c) {
    case 'c': // -c, --enable-cursor
      cursor_enabled = true;
      if (verbose) cout << "check_args: Cursor enabled" << endl;
      break;
    case 'b': // -b, --print-fps
      print_fps = true;
      if (verbose) cout << "check_args: Printing fps enabled" << endl;
      break;
    case 'l': // -l, --level
      levelfile = optarg;
      if (verbose) cout << "check_args: Levelfile = " << levelfile << endl;
      break;
    case 't': // -t, --set-speed
      game_speed = atoi(optarg);
      if (verbose) 
	cout << "check_args: Game Speed = " << game_speed << endl;
      break;
    case 's': // -s, --enable-sound
      if (verbose) cout << "check_args: Sound Effects enabled" << endl;
      cout <<
	"\n"
	"=================================================================\n"
	"                            WARNING!                             \n"
	"=================================================================\n"
	"Be warned, at the moment there is only some sound code pressent, \n"
	"there are *no* sound files, so this will fail.\n"
	"=================================================================\n" << endl;
      sound_enabled = true;
      break;
    case 'g':
      resolution = optarg;
      break;
    case 'S':
      pingus_soundfile = optarg;
      if (verbose) 
	cout << "check_args: Sound File = " << pingus_soundfile << endl;
      break;
    case 'm': // -m, --enable-music
      if (verbose) cout << "check_args: Music enabled" << endl;
      music_enabled = true;
      break;
    case 'd': // -d, --datadir
      pingus_datadir = optarg;
      pingus_datadir_set = true;
      if (verbose)
	cout << "check_args: Pingus Data Dir = " << pingus_datadir << endl;
      break;
    case 'n':
      intro_disabled = true;
      break;
    case 'V':
      cout << PACKAGE << " version " << VERSION << endl;
      cout <<
	"\n"
	"Copyright (C) 1998 Ingo Ruhnke <grumbel@gmx.de>\n"
	"There is NO warranty.  You may redistribute this software\n"
	"under the terms of the GNU General Public License.\n"
	"For more information about these matters, see the files named COPYING."
	   << endl;

      exit(EXIT_SUCCESS);
      break;
    case 'i':
      gimmicks_enabled = true;
      if (verbose) cout << "Pingus: Gimmicks enabled" << endl;
      break; 
    case 'r': // -r, --record-demo
      record_demo = true;
      demo_file = optarg;
      break;
    case 'p': // -p, --play-demo
      play_demo = true;
      demo_file = optarg;
      if (verbose)
	cout << "Using demofile: " << demo_file << endl;
      break;
    case 'v':
      sscanf(optarg, "%d", &verbose);
      cout << "Verbose level is " << verbose << endl;
      break;
    case 129: // --debug-actions
      debug_actions = true;
      break;
    case 130: // --fs-preload
      fs_preload = true;
      previews_enabled = true;
      break;
    case 132:
      fast_mode = true;
      break;
    case 133:
      previews_enabled = false;
      break;
    default:
      
      cout << "Unknow char: " << c << endl << endl;
      cout << "Usage: " << argv[0] << " [OPTIONS]... [LEVELFILE]" << endl;
      cout <<
	"\n"
	"Options:\n"
	"   -g, --geometry {width}x{height}\n"
	"                            Set the resolution for pingus (default: 640x480)\n"
	"   -h, --help               Displays this screen\n"
	"   --disable-intro          Disable intro\n"
	"   -s, --enable-sound       Enable sound\n"
	"   -m, --enable-music       Enable music\n"
	"   -i, --enable-gimmicks    Enable some buggy development stuff\n"
	"   -S, --sound-specs FILE   Use files mentioned in FILE\n"
       	"   -d, --datadir PATH       Set the path to load the data files to `path'\n"
	"   -l, --level FILE         Load a custom level from `file'\n"
	"   -t, --speed SPEED        Set the game speed (0=fastest, >0=slower)\n"
	"   -b, --print-fps          Prints the fps to stdout\n"
	"   -v, --verbose            Print some more messages to stdout, can be set\n"
	"                            multible times to increase verbosity\n"
	"   -V, --version            Prints version number and exit\n"
	"   -r, --record-demo FILE   Record a demo session to FILE\n"
	"   -p, --play-demo FILE     Plays a demo session from FILE\n"
	"   --fs-preload             Preload all Levelpreviews\n"
	"   --fast                   Disable some cpu intensive features (not implemented\n"
	"   --disable-previews       Disables all level preview in the level selector\n"
	   << endl;
      exit(EXIT_SUCCESS);
      break;
    }

  }

  // Treating non option arguments
  for(int i = optind; i < argc; ++i) {
    if (levelfile.empty()) {
      levelfile = argv[i];
    } else {
      cout << "Wrong argument: '" << argv[i] << "'" << endl;
      cout << "A levelfile is already given," << endl;
      exit(EXIT_FAILURE);
    }
  }
}

void
PingusMain::intro(void)
{
  string pingus_version;
  
  // Building version string
  pingus_version = "Pingus";
  pingus_version += " ";
  pingus_version += VERSION;

  if (verbose)
    cout << "Starting Intro" << endl;

  if (levelfile.empty()) {
    CL_Surface* sur = CL_Res_Surface::load("Game/logo", *resource);
    CL_Font*   font = CL_Res_Font::load("Fonts/smallfont", *resource);
    
    // Fade in the start screen 
    for(int i=10; i >= 0 && !CL_Keyboard::get_keycode(CL_Keyboard::KEY_ESCAPE);	--i)
      {
	if (screen_width == 640 && screen_height == 480) {
	  sur->put_screen(0,0);
	} else {
	  sur->put_screen(0,0,
			  CL_Display::get_width(), CL_Display::get_height());
	}
	CL_Display::fill_rect(0,0,
			      CL_Display::get_width(), CL_Display::get_height(),
			      0.0, 0.0, 0.0, i / 15.0);
	CL_Display::flip_display(true);
      }
      
    sur->put_screen(0,0, CL_Display::get_width(), CL_Display::get_height());
    font->print_right(CL_Display::get_width() - 20, CL_Display::get_height()-20,
		      pingus_version.c_str());
    CL_Display::flip_display(true);
    CL_Display::sync_buffers();
  }
}

// Loads a level from file and starts playing i
Result
PingusMain::play_level(string filename, PingusGameMode mode)
{
  Client*         client;
  Server* server;

  // Should contain code out of world.init
  server = new TrueServer();
  client = new Client(server);
  
  return client->start(filename, mode);
}

void
PingusMain::load_data()
{
  if (verbose) {
    cout << "Loading data... " << endl;
    cout << "Pingus Datafile: " << pingus_datafile << endl;
    cout << "Global Datafile: " << global_datafile << endl;
  }

  // This is the new costructor for ClanLib 0.1.16, not used because only in CVS
  resource = new CL_ResourceManager(global_datafile.c_str(), true);
  res_pingus = new CL_ResourceManager(pingus_datafile.c_str(), true);
  
  //  resource   = new CL_ResourceManager("global.scr", global_datafile.c_str());
  //  res_pingus = new CL_ResourceManager("pingus.scr", pingus_datafile.c_str());

  if (verbose)
    cout << "PingusMain::load_data: created resource successfully" << endl;
  
  GameObj::SetResourceManager(resource);
  Pingu::SetResourceManager(res_pingus);

  // Load the static data of some objects
  Pingu::load_data();
  Playfield::load_data();
  CaptureRectangle::load_data();

  if (verbose)
    cout << "Loading data successfull" << endl;
}

// Get all filenames and directories
void
PingusMain::get_filenames()
{
  if (verbose)
    clog << "Getting filenames..." << endl;
#ifndef WIN32
  char* env;

  char* homedir = getenv("HOME");
  if (homedir) {
    if (pingus_datadir_set)
      pingus_datadir += ":";
    
    pingus_datadir += homedir;
    pingus_datadir += "/.pingus/";
  }

  // Search different places for the datadir path, if none can be
  // found, then fail.
  // Bug: could also check for relative or absolute pathnames
  if (pingus_datadir_set) {
    pingus_datadir += ":";
    pingus_datadir += PINGUS_DATADIR;
  } else {
    env = getenv("PINGUS_DATADIR");
    
    if (env) {
      if (verbose > 1) cout << "Using envar $PINGUS_DATADIR" << endl;
      pingus_datadir = env;
      pingus_datadir_set = true;
    }

    // Correct the path name, if no slash is pressent
    //add_slash(pingus_datadir);
  
    if (exist("../data/data/global.dat")) {
      if (verbose > 1)
	cout << "Assuming that you haven't installed pingus, overriding current value." << endl;
      pingus_datadir += ":../data/";
    }

    if (!pingus_datadir_set) {
      if (verbose > 1) cout << "Using intern macro PINGUS_DATADIR" << endl;
      pingus_datadir += ":";
      pingus_datadir += PINGUS_DATADIR;
    }
  }

  if (verbose)
    cout << "pingus_datadir: " << pingus_datadir << endl;

  global_datafile   = find_file(pingus_datadir, "data/global.dat");
  pingus_datafile   = find_file(pingus_datadir, "data/pingus.dat");
  //  pingus_scriptfile = find_file(pingus_datadir, "pingus.scr");  
  // Hotspot specs will become obsolete if a level editor is available
  
  if (exist(pingus_datafile)) {
    if (verbose > 1)
      cout << "Pingus Datadir exist, all looks ok" << endl;
  } else {
    cout << "Pingus Datafile: " << pingus_datafile << endl << endl;
    cout << "Couldn't find `global.dat', please set the enviroment variable\n"
	 << "PINGUS_DATADIR to the path of the file `pingus.dat' or use the\n"
	 << "-p option." << endl;
    exit(EXIT_SUCCESS);
  } 

#else /* !WIN32 */
  //If the User uses Windows, the Datadir is always the Subdirectory "data"
  pingus_datadir_set = true;
  pingus_datadir = "data\\";
  global_datafile = pingus_datadir + "data\\global.dat";
  pingus_datafile = pingus_datadir + "data\\pingus.dat";
#endif /* !WIN32 */
  
  if (sound_enabled) {
    if (pingus_soundfile.empty())
      pingus_soundfile = find_file(pingus_datadir, "sound/sound.specs");
#ifndef WIN32
    cout << __FUNCTION__ << "PingusMain: Reading Sound Specs" << endl;
#endif /* !WIN32 */
    PingusSound::init(pingus_soundfile);
    cout << "PingusMain: Reading Sound Specs finished" << endl;
  }

  string custom_levelfile = levelfile;
  if (!levelfile.empty()) {
    if (!exist(custom_levelfile)) {
      if (verbose)
	cout << "Levelfile not found, trying fallbacks" << endl;

      // Search for the level in the datadir
      custom_levelfile = find_file(pingus_datadir, "/levels/" + custom_levelfile);
      if (!exist(custom_levelfile.c_str())) {
	cout << "Couldn't find level file: \"" << custom_levelfile << "\"" << endl;
	exit(EXIT_FAILURE);
      } 
    }
  }
  levelfile = custom_levelfile;
  
  if (verbose)
    cout << "Pingus Level File: " << levelfile << endl;
}
  
void
PingusMain::init(int argc, char* argv[])
{
  char c;

  argv_0 = argv[0];

  PingusMain::read_rc_file();
  PingusMain::check_args(argc, argv);

  if (verbose) {
    cout << PACKAGE << " version " << VERSION << endl;
  }

  // Translate the geometry string to some int's
  if (sscanf(resolution.c_str(), "%d%c%d", &screen_width, &c, &screen_height) != 3) {
    cerr << "Resolution string is wrong, it should be like: \n" 
	 << "\"640x480\" or \"800x600\"" << endl;
    exit(EXIT_FAILURE);
  }

  // Loading data and initialisising 
  get_filenames();
  //  register_actions(); 
}

void
PingusMain::init_clanlib()
{
  // Init ClanLib
  if (verbose) 
    cout << "Init ClanLib" << endl;
  CL_System::init_display();
 

  if (sound_enabled || music_enabled) {
    if (verbose)
      cout << "Init ClanSound" << endl;
    CL_System::init_sound();
  }

  if (verbose) {
    cout << "Using resolution: " 
	 << screen_width << "x" << screen_height << endl;
  }

  // Setting the display resolution
  CL_Display::set_videomode(screen_width, screen_height, 16);
}

bool
PingusMain::do_lemmings_mode(void)
{
  Result result;

  if (verbose)
    cout << "Starting Main: " << CL_System::get_time() << endl;
  
  /*
    if (levelfile == "")
    { 
      PingusMenu menu;
      
      switch(menu.select())
	{ 
	case PingusMenu::START:
	  cout << "PingusMenu::Start" << endl;
	  break;

	case PingusMenu::LOAD:
	  cout << "PingusMenu::Load" << endl;
	  break;

	case PingusMenu::QUIT:
	  cout << "PingusMenu::Quit" << endl;
	  break;

	default: cout << "PingusMenu::Fatal Error" << endl; assert(0);
	  break;
	}
    }
  */
  
  do {
    if (levelfile.empty()){
      FileSelector file;
      levelfile = file.select(pingus_datadir, "*.plf");

      if (levelfile.empty()) {
	cout << "No File Selected, going down" << endl;
	break;
      }
    }
      
    if (verbose) cout << "Levelfile: " << levelfile << endl;
    
    try {
      result = play_level(levelfile, INTERACTIVE_MODE);
      {
	if (verbose) cout << "Displaying Level Result" << endl;
	PingusLevelResult res(result);
	res.draw();
      }
    }

    catch (CL_Error err) {
      PingusMessageBox(string(" CL_Error: ") + err.message.get_string());
      result.next_action(LevelInterrupt(LevelInterrupt::select_new));
    }

    catch (PingusError err) {
      PingusMessageBox(" PingusError: " + err.message);
      result.next_action(LevelInterrupt(LevelInterrupt::select_new));
    }
  
    levelfile = "";
  } while (result.next_action().get_status() == LevelInterrupt::select_new);

  cout << "Pingus exit normaly" << endl;
  result.print();
  
  return true;
}

bool
PingusMain::do_worms_mode(void)
{
  cout << "do_worms_mode() not implemented" << endl;
  return false;
}

PingusMain::GameMode
PingusMain::select_game_mode(void)
{
  // dummy, should replace with a nice menu
  return Lemmings;
}

int
PingusMain::main(int argc, char** argv)
{
  bool quit = false;

  try {
    init(argc, argv);
    init_clanlib();	
    load_data();	

    if (!intro_disabled)
      intro();

    while (!quit) {
      switch (select_game_mode()) {
      case PingusMain::Lemmings: // Select the Lemmings(tm) like mode
	quit = do_lemmings_mode();
	break;
      case PingusMain::Worms:  // Select the Worms(tm) like mode
	quit = do_worms_mode();
	break;
      }
    }
  }
  
  catch (CL_Error err) {
    cout << "Error caught from ClanLib: " << err.message << endl;
  }

  catch (PingusError err) {
    cout << "Error caught from Pingus: " << err.message << endl;
  }

  catch (...) {
    cout << "Pingus: Unknown throw catched!" << endl;
  }

  return 0;
}

void
PingusMain::remove_comments(char* line)
{
  for (int i=0; line[i] != '\0'; ++i) {
    if (line[i] == '#') {
      line[i] = '\0';
      break;
    }
  }
}

bool 
PingusMain::line_empty(char* line)
{
  for (int i=0; line[i] != '\0'; ++i) {
    if (!isspace(line[i]))
      return false;
  }
  return true;
}

/* EOF */
