// Pingus - A free Lemmings clone
// Copyright (C) 1999 Ingo Ruhnke <grumbel@gmx.de>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifndef PLAYFIELD_HH
#define PLAYFIELD_HH

#include <ClanLib/clanlib.h>

#include "Range.hh"
#include "ButtonPanel.hh"
#include "GuiObj.hh"
#include "World.hh"
#include "GameObj.hh"
#include "View.hh"
#include "PinguInfo.hh"
#include "Server.hh"

class Playfield : public GuiObj, public GameObj
{
private:  
  static CL_Surface* pause_s;
  static AnimCounter pause_c;

  CL_InputSourceProvider* datafile;
  CL_Surface* buffer;
  Server* server;
  Range x_offset2, y_offset2;
  ButtonPanel* buttons;
  PinguHolder* pingus;
  PinguInfo* pingu_info;
  World* world;
  vector<View> view;
  
  int  x_offset, y_offset;
  Pingu* current_pingu;
  bool mouse_scrolling;
  bool pause;
  int current_view;
  int scroll_speed;
public:
  Playfield();
  Playfield(PLF*, World*);
  ~Playfield();

  void draw();
  void let_move();
  //  void write_results();
  void process_input();
  void process_input_interactive();
  void process_input_demomode();
  void set_world(World*);
  static void load_data();
  Pingu* current_pingu_find(int x, int y, double s);
  bool enough_time_passed();

  // Members used to communicate between different screen objs
  void set_pingu_info(PinguInfo*);
  void set_buttons(ButtonPanel* b);
  void set_server(Server*);
};

#endif

/* EOF */
