// Pingus - A free Lemmings clone
// Copyright (C) 1999 Ingo Ruhnke <grumbel@gmx.de>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "hammer.hh"
#include "../particles/GroundParticle.hh"
#include "../algo.hh"
#include "../actions/smashed.hh"

Hammer::Hammer(trap_data data)
{
  x_pos = data.x_pos;
  y_pos = data.y_pos;
  z_pos = data.z_pos;
  surface = CL_Res_Surface::load("Traps/hammer", *resource());
  counter.set_size(surface->get_no_sprs());
  counter.set_type(GameCounter::ping_pong);
  counter.set_speed(1);
}

Hammer::~Hammer()
{
}

void
Hammer::let_move()
{
  if (counter == 0) 
    particle_thrown = false;

  if (counter == surface->get_no_sprs() - 3 && !particle_thrown) {
    particle_thrown = true;
    for(int i=0; i < 5; ++i)
      particle->add_particle(new GroundParticle(x_pos + 67 + rand() % 40 - 20 ,
						y_pos + 177,
						frand() * 2 - 1,
						frand() * - 1.5));
  }
  ++counter;
}

bool
Hammer::catch_pingu(Pingu* pingu)
{
  if (counter >= surface->get_no_sprs() - 3) {
    if (pingu->get_x() > x_pos + 60 && pingu->get_x() < x_pos + 100
	&& pingu->get_y() > y_pos + 160 && pingu->get_y() < y_pos + 180)
      pingu->set_action(new Smashed);
    return true;
  } else {
    return false;
  }
}

/* EOF */
